const util = require("../../utils/util");
const {inArray, ab2hex} = util

const plugin = requirePlugin("sdkPlugin").AiLink;


Page({
  data: {
    showWriteInput: false,
    devices: [
      // {
      //   deviceId: "02:03:04:05:06:07",
      //   name: "elink",
      //   localName: "elink",
      //   mac: "02:03:04:05:06:07",
      //   RSSI: -69,
      //   advertisServiceUUIDs:[
      //     "FFE0","FFC0"
      //   ],
      //   analyzeDataText:"0102003241123413"
      // }
    ],
    connected: false,
    chs: [],
    cmd: '',
    name: '',
    deviceId: null,
    historyList: [],
  },
  onLoad: function () {
    let historyList = []
    historyList = wx.getStorageSync('historyList')
    this.setData({
      historyList,
    })
  },

  toConnectWifi() {
    this.offBLECharacteristicValueChange()
    wx.navigateTo({
      url: '../wifiList/wifiList',
    })
  },

  // 初始化蓝牙模块
  openBluetoothAdapter() {
    wx.openBluetoothAdapter({
      success: (res) => {
        console.log('openBluetoothAdapter success', res)
        this.startBluetoothDevicesDiscovery()
      },
      fail: (res) => {
        if (res.errCode === 10001) {
          wx.showToast({
            title: '请打开蓝牙',
            icon: "none"
          })
          wx.onBluetoothAdapterStateChange( (res)=> {
            console.log('onBluetoothAdapterStateChange', res)
            if (res.available) {
              this.startBluetoothDevicesDiscovery()
            }
          })
        }
      }
    })
  },
  // 获取本机蓝牙适配器状态
  getBluetoothAdapterState() {
    wx.getBluetoothAdapterState({
      success: (res) => {
        console.log('getBluetoothAdapterState', res)
        if (res.discovering) {
          this.onBluetoothDeviceFound()
        } else if (res.available) {
          this.startBluetoothDevicesDiscovery()
        }
      }
    })
  },
  // 开始搜寻附近的蓝牙外围设备
  startBluetoothDevicesDiscovery() {
    if (this._discoveryStarted) {
      return
    }
    this._discoveryStarted = true
    wx.startBluetoothDevicesDiscovery({
      allowDuplicatesKey: true,
      services: [
        "FFE0",
      ],
      success: (res) => {
        console.log('startBluetoothDevicesDiscovery success', res)
        this.onBluetoothDeviceFound()
      },
    })
  },
  // 停止搜寻附近的蓝牙外围设备
  stopBluetoothDevicesDiscovery() {
    wx.stopBluetoothDevicesDiscovery()
  },

  // 监听寻找到新设备的事件
  onBluetoothDeviceFound() {
    wx.onBluetoothDeviceFound((res) => {
      res.devices.forEach(device => {
        if (!device.name && !device.localName) {
          return
        }
        const foundDevices = this.data.devices
        const idx = inArray(foundDevices, 'deviceId', device.deviceId)
        const data = {}
        // console.log(device)
        // console.log(ab2hex(device.advertisData))
        
          let buff = device.advertisData.slice(-6)
          device.mac = new Uint8Array(buff) // 保存广播数据中的mac地址，这是由于iOS不直接返回mac地址
          let tempMac = Array.from(device.mac)
          tempMac.reverse()
          device.macAddr = ab2hex(tempMac, ':').toUpperCase()
        
        if (idx === -1) {
          data[`devices[${foundDevices.length}]`] = device
        } else {
          data[`devices[${idx}]`] = device
        }
        this.setData(data)
      })
    })
  },
  // 连接低功耗蓝牙设备
  createBLEConnection(e) {
    this._connLoading = true
    wx.showLoading({
      title: '连接中',
    })
    setTimeout(() => {
      if (this._connLoading) {
        this._connLoading = false
        wx.hideLoading()
      }
    }, 6000)
    const ds = e.currentTarget.dataset
    const index = ds.index
    // 保存当前连接的设备，注意不能从wxml的dataset中直接返回该对象，因为ArrarBuffer类型的数据无法保留
    this._device = this.data.devices[index]
    console.log(this._device)
    const deviceId = ds.deviceId
    const name = ds.name
    this.mac = ds.mac
    wx.createBLEConnection({
      deviceId,
      success: (res) => {
        this.setData({
          connected: true,
          name,
          deviceId,
        })
        console.log("createBLEConnection:success")
        wx.stopBluetoothDevicesDiscovery()
        this.onBLEConnectionStateChange()
        this.getBLEDeviceServices(deviceId)
      },
      fail: res => {
        this._connLoading = false
        wx.hideLoading()
        wx.showToast({
          title: '连接失败',
          icon: 'none'
        })
      }
    })
    // 连接上设备就可以停止蓝牙搜索，减少功耗。
    this.stopBluetoothDevicesDiscovery()
  },

  onBLEConnectionStateChange() {
    wx.onBLEConnectionStateChange((res) => {
      console.log('wx.onBLEConnectionStateChange() ', res.deviceId, res.connected, res.errorCode, res.errorMsg)
      // 该方法回调中可以用于处理连接意外断开等异常情况
      // console.log(`%c device ${res.deviceId} state has changed, connected: ${res.connected}`, 'color: #F26363')
      if (!res.connected) {
        setTimeout(()=>{
          wx.showToast({
            title: '连接已断开',
            icon: 'none'
          })
        },500)
        wx.navigateBack()
        this.setData({
          connected: false,
          showWriteInput: false,
        })
      }
    })
  },

  // 断开与低功耗蓝牙设备的连接
  closeBLEConnection() {
    wx.closeBLEConnection({
      deviceId: this._deviceId
    })
    this.setData({
      connected: false,
      chs: [],
      showWriteInput: false,
    })
  },

  // 获取蓝牙设备的 serviceId
  getBLEDeviceServices(deviceId) {
    wx.getBLEDeviceServices({
      deviceId,
      success: (res) => {
        for (let i = 0; i < res.services.length; i++) {
          if (res.services[i].isPrimary && res.services[i].uuid.indexOf('FFE0') > -1) {
            this.getBLEDeviceCharacteristics(deviceId, res.services[i].uuid)
            return
          }
        }
      }
    })
  },

  // 获取蓝牙设备某个服务中所有特征值(characteristic)
  getBLEDeviceCharacteristics(deviceId, serviceId) {
    this._deviceId = deviceId
    this._serviceId = serviceId
    this._device.serviceId = serviceId

    wx.getBLEDeviceCharacteristics({
      deviceId,
      serviceId,
      success: (res) => {
        console.log('getBLEDeviceCharacteristics success', res.characteristics)

        // 初始化插件
        plugin.initPlugin(res.characteristics, this._device)

        wx.onBLECharacteristicValueChange((characteristic) => {
          // 解析特征值，返回解密后的数据
          let bleData = plugin.parseBleData(characteristic.value)
          console.log(bleData)
          if (bleData.status == 0) {
            console.log("握手成功")
            this._connLoading = false
            wx.hideLoading()
            wx.showToast({
              title: '连接成功',
            })
            setTimeout(()=>{
              this.toConnectWifi()
            },500)
          } else if (bleData.status == 1) {
            console.log('A7-----^')
          } else if (bleData.status == 2) {
            console.log('A6-----^')
          }
        })
      },
      fail(res) {
        console.error('getBLEDeviceCharacteristics', res)
      }
    })
  },
  writeBLECharacteristicValue(buffer, uuid, deviceId, serviceId) {
    // 向蓝牙设备发送一个二进制流数据
    wx.writeBLECharacteristicValue({
      deviceId,
      serviceId,
      characteristicId: uuid,
      value: buffer,
      success(res) {
        console.log('writeBLECharacteristicValue success', res)
        console.log('下发指令==> ' + ab2hex(buffer))
      }
    })
  },

  offBLECharacteristicValueChange() {
    wx.offBLECharacteristicValueChange((res)=>{
      console.log(res)
    })
  },
  closeBluetoothAdapter() {
    wx.closeBluetoothAdapter()
    this._discoveryStarted = false
  },
  // 打开指令输入框
  showWriteInputView() {
    this.setData({
      showWriteInput: true
    })
  },
  // 关闭指令输入框
  hideWriteInputView() {
    this.setData({
      showWriteInput: false
    })
  },
  // 指令下发
  submitCmd(e, cmd) {
    let arr = []
    let temp = []
    if (!cmd) {
      cmd = this.data.cmd
    }
    console.log(cmd)
    if(!cmd || !this.data.connected){
      return
    }
    if(cmd.indexOf(",") == -1){
      temp = cmd.split(" ")
    } else {
      temp = cmd.split(",")
    }
    let tempCmd = temp.join(' ')
    for(let i = 0; i < temp.length; i++){
      arr[i] = parseInt(temp[i],16)
    }
    // let arr = [
    //   0xA6,
    //   0x01,
    //   0x28,
    //   0x29,
    //   0x6A,
    //   //A6 01 28 29 6A
    // ]
    if(arr[0] == 0xA6){
      let len = arr[1]
      let payload = arr.slice(2, 2 + len)
      plugin.sendDataOfA6(payload)
      // let buff = new Uint8Array(arr).buffer
      // this.writeBLECharacteristicValue(buff, this.uuid3, this._deviceId, this._serviceId)
    } else if(arr[0] == 0xA7) {
      let len = arr[3]
      let payload = arr.slice(4, 4 + len)
      plugin.sendDataOfA7(payload)
    }
    let historyList = wx.getStorageSync('historyList') || []
    let idx = historyList.findIndex(item => item.cmd == tempCmd)
    if (idx < 0) {
      historyList.push({cmd: tempCmd})
    }
    this.setData({
      historyList
    })
    wx.setStorage({
      data: historyList,
      key: 'historyList',
    })
    wx.showToast({
      title: '已发送',
      icon: 'none'
    })
  },
  history_delete(e) {
    let index = e.currentTarget.dataset.index
    this.data.historyList.splice(index, 1)
    this.setData({
      historyList: this.data.historyList
    })
    wx.setStorage({
      data: this.data.historyList,
      key: 'historyList',
    })
  },
  history_copy(e) {
    let index = e.currentTarget.dataset.index
    wx.setClipboardData({
      data: this.data.historyList[index].cmd
    })
  },
  history_send(e) {
    let index = e.currentTarget.dataset.index
    let cmd = this.data.historyList[index].cmd
    this.submitCmd(null, cmd)
  },
});