// pages/wifiList/wifiList.js
const util = require("../../utils/util");
const {
  ab2hex
} = util;

const plugin = requirePlugin("sdkPlugin").AiLink;

Page({

  /**
   * 页面的初始数据
   */
  data: {
    list: [],
    wifi_connected: false,
    conn_wifi: {},
    scanEnd: false,
    totalAP: 0,
    pwdModal: false,
    pwdInput: '',
    statusText: '',
    statusList: ['', '', '已连接']
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad: function (options) {
    let pages = getCurrentPages()
    this._home = pages[pages.length - 2]

    wx.onBLECharacteristicValueChange((characteristic) => {
      // 解析特征值，返回解密后的数据
      let bleData = plugin.parseBleData(characteristic.value)
      // console.log(bleData)
      if (bleData.status == 0) {
        console.log("握手成功")
      } else if (bleData.status == 1) {
        console.log('A7-----^')
      } else if (bleData.status == 2) {
        console.log(bleData)
        console.log('A6-----^')
        let payload = bleData.data
        const result = ['成功', '失败', '不支持']
        switch (payload[0]) {
          case 0x26: {
            console.log('状态改变')
            let bleStatus = payload[1] & 0x0f
            let wifiStatus = payload[1] >> 4
            const bleStatusList = ['无连接', '已连接', '配对完成']
            const wifiStatusList = ['未连接', '连接失败', '连接的WiFi信号不好', '连接成功', '正在连接']
            console.log('蓝牙状态：' + bleStatusList[bleStatus])
            console.log('WiFi状态：' + wifiStatusList[wifiStatus])
            this.setData({
              statusText: '蓝牙状态：' + bleStatusList[bleStatus] + ' ，WiFi状态：' + wifiStatusList[wifiStatus]
            })
            break
          }
          case 0x80: {
            wx.hideLoading()
            wx.showToast({
              title: '扫描AP' + result[payload[1]],
              icon: 'none'
            })
            console.log('扫描AP' + result[payload[1]])
            break;
          }
          case 0x81: {
            let id = payload[1]
            console.log('上报WiFi名===>')
            let nameArr = payload.slice(2)
            console.log('原始数据===>' + ab2hex(nameArr, ' '))
            let name = nameArr.length ? util.hex2str(nameArr) : ''
            if (name) {
              console.log('WiFi名===>' + name)
              let index = this.data.list.findIndex(item => item.name == name)
              if (index < 0) {
                this.data.list.push({
                  id,
                  name,
                })
              } else {
                this.data.list[index].name = name
              }
              this.setData({
                list: this.data.list
              })
            }
            break;
          }
          case 0x82: {
            console.log('上报WiFi信息-----^')
            let id = payload[1]
            let index = this.data.list.findIndex(item => item.id == id)
            let addr = payload.slice(2, 8)
            let signal = payload[8]
            let type = payload[9]
            let status = payload[10]
            if (index < 0) {
              // this.data.list.push({
              //   id,
              //   addr,
              //   signal,
              //   type,
              //   status,
              // })
            } else {
              this.data.list[index].addr = addr
              this.data.list[index].signal = signal
              this.data.list[index].type = type
              this.data.list[index].status = status
              this.setData({
                list: this.data.list
              })
            }
            break;
          }
          case 0x83:
            let num = payload[1]
            console.log('上报扫描完毕-----^')
            this.setData({
              totalAP: num,
              scanEnd: true
            })
            break;
          case 0x84: {
            console.log('设置地址' + result[payload[1]])
            wx.showToast({
              title: '设置地址' + result[payload[1]],
              icon: 'none'
            })
            // if (this._conn_wifi.status > 0) {
            //   this.connectWifi()
            // }
            break;
          }
          case 0x86: {
            console.log('设置密码' + result[payload[1]])
            wx.showToast({
              title: '设置密码' + result[payload[1]],
              icon: 'none'
            })
            if (payload[1] === 0) {
              this.connectWifi()
              this.closeModal()
            }
            break;
          }
          case 0x88: {
            console.log('设置 wifi 状态' + result[payload[1]])
            wx.showToast({
              title: '设置 wifi 状态' + result[payload[1]],
              icon: 'none'
            })
            if (payload[1] === 0) {
              if (this.data.wifi_connected) { // true 建立连接， false 断开连接
                this.data.list.forEach(item=>{
                  if (item.name === this._conn_wifi.name) {
                    item.status = 2
                  } else if (item.status == 2) {
                    item.status = 1
                  }
                })
              }
              this.setData({
                conn_wifi: this._conn_wifi,
                wifi_connected: this.data.wifi_connected,
                list: this.data.list
              })
            } else {
              this.setData({
                wifi_connected: !this.data.wifi_connected
              })
            }
            break;
          }

        }
      }
    })
  },

  doSearch() {
    let payload = [0x80, 0x01]
    this.setData({
      scanEnd: false,
    })
    wx.showLoading({
      title: '开始搜索',
    })
    this.data.list = []
    plugin.sendDataOfA6(payload)
  },

  tapItem(e) {
    let index = e.currentTarget.dataset.index
    let item = this.data.list[index]
    console.log(item)
    // 设置希望连接的 AP 地址
    let cmd = [0x84, ...item.addr]
    plugin.sendDataOfA6(cmd)
    this.showModal()
    this._conn_wifi = item
  },

  confirmPwd() {
    if (!this.data.pwdInput || this.data.pwdInput.length < 8) {
      wx.showToast({
        title: '密码有误',
        icon: 'none'
      })
      return
    }
    let pwdArr = util.str2hex(this.data.pwdInput)
    if (pwdArr.length > 56) {
      wx.showToast({
        title: '密码有误',
        icon: 'none'
      })
      return
    }
    if (pwdArr.length > 14) { // 超出14 byte 需分包
      console.log('密码超出14位===')
      let pwdList = []
      let len = parseInt(pwdArr.length / 14) + 1
      for (let i = 0; i < len; i++) {
        setTimeout(() => {
          pwdList[i] = pwdArr.slice(i * 14, (i + 1) * 14)
          let isEnd = +(i !== len - 1)
          console.log(isEnd)
          let cmd = [0x86, isEnd, ...pwdList[i]]
          console.log(cmd)
          plugin.sendDataOfA6(cmd)
        }, 200 * i)
      }
      console.log(pwdArr)
      console.log(pwdList)
    } else {
      // 设置希望连接的 AP 密码
      let cmd = [0x86, 0x00, ...pwdArr]
      plugin.sendDataOfA6(cmd)
    }
  },

  connectWifi() {
    // 发起 wifi 连接
    this.setData({
      wifi_connected: true
    })
    let cmd = [0x88, 0x01]
    plugin.sendDataOfA6(cmd)
  },

  closeWifiConnection() {
    // 断开 wifi 连接
    this.setData({
      wifi_connected: false
    })
    let cmd = [0x88, 0x00]
    plugin.sendDataOfA6(cmd)
  },

  showModal() {
    this.setData({
      pwdModal: true
    })
  },

  closeModal() {
    this.setData({
      pwdModal: false
    })
  },

  stopAction() {
    return false
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady: function () {

  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow: function () {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide: function () {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload: function () {

  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh: function () {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom: function () {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage: function () {

  }
})